package com.itbc.lucene;


import java.util.ArrayList; 

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.Term;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.itbc.Element.*;

public class IndexEngine {
	
	private static IndexEngine instance = null;//new IndexEngine();
	private static Analyzer analyzer = null;//new KoreanAnalyzer();//new StandardAnalyzer()
	private Object lock = new Object();
	private String indexFolder = null;
	private LuceneManager manager;
	
	public static IndexEngine getInstance() {
		return instance;
	}
 
	//public IndexEngine() {}	
	public IndexEngine(String root) {
		analyzer  = new KoreanAnalyzer(root);
	}	
 /*
	public void initialize() {

		synchronized(lock) {
			Directory directory = null;
			IndexWriter writer = null;
			try {			
				directory = FSDirectory.getDirectory(indexFolder, false);
				if (!directory.fileExists("segments")) 
					writer = new IndexWriter(indexFolder, analyzer, true);
			} catch (Exception exception) {
			} finally {
				if (writer != null) try { writer.close(); } catch (Exception exception) {}
				if (directory != null) try { directory.close(); } catch (Exception exception) {}
			}
		}
	}
	*/
	public void reorgainze() throws Exception {

		synchronized(lock) {			
			IndexWriter writer = null;
			try {
				System.out.println("re : "+indexFolder);
				writer = new IndexWriter(indexFolder, analyzer, true);
			    int mergeFactor = 100;
//			    int maxMergeDocs = 10000;
			    
			    writer.setMergeFactor(mergeFactor);
			    
//			    writer.setMaxMergeDocs(maxMergeDocs);
			    

			    reorganizeIndex(writer);
			} catch (Exception e) {
				throw e;
			} finally {
				if (writer != null){
					try { 
						writer.close(); 
					} catch (Exception exception) {}
				}
			}
		}
		
	}
	
	public void indexing(NoticeElement bean) throws Exception {
	System.out.println("indexing ");	
		synchronized(lock) {
			IndexWriter writer = null;			
			try {
				System.out.println("folder : "+indexFolder);
				writer = new IndexWriter(indexFolder, analyzer, false);
 
				insertIndex(writer, bean);
			}catch (Exception e) {
				System.out.println("indexing err : "+e);
			} finally {
				if (writer != null) try { writer.close(); } catch (Exception exception) {}
			}
		}
		
	}
	
	public void deletes(String seq) throws Exception {
 
		synchronized(lock) {
			IndexReader reader = null;
			Directory directory = null;			
			try {
				directory = FSDirectory.getDirectory(indexFolder, false);
				reader = IndexReader.open(directory);
				Term term = null;				
				term = new Term("seq", seq);
				reader.deleteDocuments(term);
				reader.close();
			} catch (Exception e) {
			} finally {
				if (reader != null) try { reader.close(); } catch (Exception exception) {}
				if (directory != null) try { directory.close(); } catch (Exception exception) {}
			}
		}
		
	}
	
	public void update(NoticeElement bean) throws Exception {
		synchronized(lock) {
			IndexReader reader = null;
			Directory directory = null;			
			try {
				directory = FSDirectory.getDirectory(indexFolder, false);
				reader = IndexReader.open(directory);
				
				String seq = null;
				Term term = null;				

				seq = ""+bean.getNotice_seq();
				term = new Term("seq", seq); 
				
				reader.deleteDocuments(term);
				reader.close();
				indexing(bean);
			} catch (Exception e) {
			} finally {
				if (reader != null) try { reader.close(); } catch (Exception exception) {}
				if (directory != null) try { directory.close(); } catch (Exception exception) {}
			}
		}
		
	}
	
	public void reorganizeIndex(IndexWriter writer) throws Exception {
		System.out.println("reorganizeIndex() : "+writer);
		try {
			manager = new LuceneManager();
			ArrayList result = null;
			int num=0;
			while (true) {
				result = manager.getallList(num);
				
				Document document = null;
				NoticeElement bean = null;
				System.out.println("result.size() : "+result.size());
				if(result.size()==0){
					document = new Document();
					document.add(new org.apache.lucene.document.Field
							("seq",  "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
					document.add(new org.apache.lucene.document.Field("title", "", org.apache.lucene.document.Field.Store.YES,org.apache.lucene.document.Field.Index.TOKENIZED));
					document.add(new org.apache.lucene.document.Field("content", "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.TOKENIZED));
					document.add(new org.apache.lucene.document.Field("division",  "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
					document.add(new org.apache.lucene.document.Field("date", "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
					//document.add(new org.apache.lucene.document.Field("file1", "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//					document.add(new org.apache.lucene.document.Field("smallImage", "", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
					writer.addDocument(document);
					
				}else{
				
					for (int i = 0; i < result.size(); i++) {
						
						document = new Document();
						bean = (NoticeElement)result.get(i);
						System.out.println("bean : "+bean.getNotice_title());
						//String title = new String(bean.getNotice_title().getBytes("utf-8"),"euc-kr");
						document.add(new org.apache.lucene.document.Field("seq",  ""+bean.getNotice_seq(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
						document.add(new org.apache.lucene.document.Field("title", bean.getNotice_title(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.TOKENIZED));
						document.add(new org.apache.lucene.document.Field("content", bean.getNotice_content(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.TOKENIZED));
						document.add(new org.apache.lucene.document.Field("division",  bean.getNotice_division(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
						document.add(new org.apache.lucene.document.Field("date", bean.getNotice_date(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//						document.add(new org.apache.lucene.document.Field("file1", (bean.getFile1()!=null)?bean.getFile1():"", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//						document.add(new org.apache.lucene.document.Field("smallImage", (bean.getSmallImage()!=null)?bean.getSmallImage():"samallTest.gif", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
						writer.addDocument(document);
						System.out.println("count : "+i);
					}
				}
				num++;
				if (result.size() == 0 || result.size() < 1000){
					break;
				}
			}
			result.clear();
			result = null;
			writer.optimize();

		} catch (Exception e) {
			System.out.println("reorganizeIndex() error : "+e);
		} finally {
		}
		
	}

	
	public void insertIndex(IndexWriter writer, NoticeElement bean) throws Exception {
		System.out.println("insertIndex");
		try {	
			Document document = new Document();
			document.add(new org.apache.lucene.document.Field("seq",  ""+bean.getNotice_seq(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
			document.add(new org.apache.lucene.document.Field("title", bean.getNotice_title(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.TOKENIZED));
			document.add(new org.apache.lucene.document.Field("content", bean.getNotice_content(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.TOKENIZED));
			document.add(new org.apache.lucene.document.Field("division",  bean.getNotice_division(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
			document.add(new org.apache.lucene.document.Field("date", bean.getNotice_date(), org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//			document.add(new org.apache.lucene.document.Field("file1", (bean.getFile1()!=null)?bean.getFile1():"", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//			document.add(new org.apache.lucene.document.Field("image", (bean.getImage()!=null)?bean.getImage():"test.gif", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
//			document.add(new org.apache.lucene.document.Field("smallImage", (bean.getSmallImage()!=null)?bean.getSmallImage():"samallTest.gif", org.apache.lucene.document.Field.Store.YES, org.apache.lucene.document.Field.Index.UN_TOKENIZED));
 
			writer.addDocument(document);
//			writer.optimize();

		} catch (Exception e) {
			e.printStackTrace();
		} finally {
		}
	}
	
	
	public void setIndexFolder(String indexFolder) {
		this.indexFolder = indexFolder; 
	}
	
	public String getIndexFolder() {
		return indexFolder;
	}
}